'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getField = exports.initialFormState = exports.initialFieldState = exports.formReducer = exports.createFormReducer = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _every = require('lodash/every');

var _every2 = _interopRequireDefault(_every);

var _icepick = require('icepick');

var _icepick2 = _interopRequireDefault(_icepick);

var _isBoolean = require('lodash/isBoolean');

var _isBoolean2 = _interopRequireDefault(_isBoolean);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _mapValues = require('lodash/mapValues');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _toPath = require('lodash/toPath');

var _toPath2 = _interopRequireDefault(_toPath);

var _flat = require('flat');

var _flat2 = _interopRequireDefault(_flat);

var _actionTypes = require('../action-types');

var _actionTypes2 = _interopRequireDefault(_actionTypes);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var initialFieldState = {
  blur: true, // will be deprecated
  dirty: false, // will be deprecated
  focus: false,
  pending: false,
  pristine: true,
  submitted: false,
  touched: false,
  untouched: true, // will be deprecated
  valid: true,
  validating: false,
  viewValue: null,
  validity: {},
  errors: {}
};

var initialFormState = _extends({}, initialFieldState, {
  fields: {},
  submitFailed: false,
  retouched: false
});

function getField(state, path) {
  if (!(0, _isPlainObject2.default)(state) || !state.fields) {
    throw new Error('Error when trying to retrieve field \'' + path + ' from an invalid/empty form state. Must pass in a valid form state as the first argument.');
  }

  var localPath = typeof path === 'function' ? path(state) : (0, _toPath2.default)(path);

  if (!localPath.length) {
    return state;
  }

  return (0, _get3.default)(state, ['fields', localPath.join('.')], initialFieldState);
}

function setField(state, localPath, props) {
  if (!localPath.length) {
    return _icepick2.default.merge(state, props);
  }

  return _icepick2.default.merge(state, {
    fields: _defineProperty({}, localPath.join('.'), _extends({}, getField(state, localPath), props))
  });
}

function resetField(state, localPath) {
  return _icepick2.default.setIn(state, ['fields', localPath.join('.')], initialFieldState);
}

function formIsValid(formState) {
  return (0, _every2.default)((0, _mapValues2.default)(formState.fields, function (field) {
    return field.valid;
  })) && (0, _every2.default)(formState.errors, function (error) {
    return !error;
  });
}

function createInitialFormState(model, initialState) {
  var formState = _icepick2.default.set(initialFormState, 'model', model);

  if (initialState) {
    return _icepick2.default.set(formState, 'fields', (0, _mapValues2.default)((0, _flat2.default)(initialState), function (initialValue) {
      return _icepick2.default.set(initialFieldState, 'initialValue', initialValue);
    }));
  }

  return formState;
}

function _createFormReducer(model, initialState) {
  var modelPath = (0, _toPath2.default)(model);
  var localInitialFormState = createInitialFormState(model, initialState);

  var formReducer = function formReducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? localInitialFormState : arguments[0];
    var action = arguments[1];

    if (!action.model) {
      return state;
    }

    var path = (0, _toPath2.default)(action.model);

    if (!(0, _isEqual2.default)(path.slice(0, modelPath.length), modelPath)) {
      return state;
    }

    var localPath = path.slice(modelPath.length);
    var errors = void 0;
    var validity = void 0;

    switch (action.type) {
      case _actionTypes2.default.BATCH:
        return action.actions.reduce(formReducer, state);

      case _actionTypes2.default.FOCUS:
        return setField(state, localPath, {
          blur: false, // will be deprecated
          focus: true
        });

      case _actionTypes2.default.CHANGE:
        {
          if (action.silent) return state;

          var setDirtyState = _icepick2.default.merge(state, {
            dirty: true, // will be deprecated
            pristine: false
          });

          return setField(setDirtyState, localPath, {
            dirty: true, // will be deprecated
            pristine: false
          });
        }

      case _actionTypes2.default.SET_DIRTY:
        {
          var _setDirtyState = _icepick2.default.merge(state, {
            dirty: true, // will be deprecated
            pristine: false
          });

          return setField(_setDirtyState, localPath, {
            dirty: true, // will be deprecated
            pristine: false
          });
        }

      case _actionTypes2.default.BLUR:
      case _actionTypes2.default.SET_TOUCHED:
        {
          var fieldState = setField(state, localPath, {
            focus: false,
            touched: true,
            retouched: state.submitted || state.submitFailed,
            blur: true, // will be deprecated
            untouched: false });

          // will be deprecated
          return _icepick2.default.merge(fieldState, {
            touched: true,
            untouched: false });
        }

      // will be deprecated
      case _actionTypes2.default.SET_PENDING:
        return setField(state, localPath, {
          pending: action.pending,
          submitted: false,
          submitFailed: false,
          retouched: false
        });

      case _actionTypes2.default.SET_VALIDITY:
        {
          if ((0, _isPlainObject2.default)(action.validity)) {
            validity = _icepick2.default.merge(getField(state, localPath).validity, action.validity);

            errors = _extends({}, getField(state, localPath).errors, (0, _mapValues2.default)(action.validity, function (valid) {
              return !valid;
            }));
          } else {
            validity = action.validity;
            errors = !action.validity;
          }

          var formIsValidState = setField(state, localPath, {
            errors: errors,
            validity: validity,
            valid: (0, _isBoolean2.default)(errors) ? !errors : (0, _every2.default)(errors, function (error) {
              return !error;
            })
          });

          return _icepick2.default.merge(formIsValidState, {
            valid: formIsValid(formIsValidState)
          });
        }

      case _actionTypes2.default.SET_FIELDS_VALIDITY:
        return (0, _map2.default)(action.fieldsValidity, function (fieldValidity, field) {
          return {
            type: _actionTypes2.default.SET_VALIDITY,
            model: model + '.' + field,
            validity: fieldValidity,
            options: action.options
          };
        }).reduce(formReducer, state);

      case _actionTypes2.default.SET_ERRORS:
        {
          if ((0, _isPlainObject2.default)(action.errors)) {
            validity = _extends({}, getField(state, localPath).validity, (0, _mapValues2.default)(action.errors, function (error) {
              return !error;
            }));

            errors = _icepick2.default.merge(getField(state, localPath).errors, action.errors);
          } else {
            validity = !action.errors;
            errors = action.errors;
          }

          var setErrorsState = setField(state, localPath, {
            errors: errors,
            validity: validity,
            valid: (0, _utils.isValid)(validity)
          });

          return _icepick2.default.merge(setErrorsState, {
            valid: formIsValid(setErrorsState)
          });
        }

      case _actionTypes2.default.RESET_VALIDITY:
        {
          var _ret = function () {
            var resetValidityState = void 0;
            if (!localPath.length) {
              resetValidityState = _icepick2.default.setIn(state, ['valid'], true);

              resetValidityState = _icepick2.default.setIn(resetValidityState, ['validity'], initialFieldState.validity);

              resetValidityState = _icepick2.default.setIn(resetValidityState, ['errors'], initialFieldState.errors);

              Object.keys(resetValidityState.fields).forEach(function (field) {
                resetValidityState = _icepick2.default.setIn(resetValidityState, ['fields', field, 'valid'], true);

                resetValidityState = _icepick2.default.setIn(resetValidityState, ['fields', field, 'validity'], initialFieldState.validity);

                resetValidityState = _icepick2.default.setIn(resetValidityState, ['fields', field, 'errors'], initialFieldState.errors);
              });
            } else {
              resetValidityState = _icepick2.default.setIn(state, ['fields', localPath.join('.'), 'valid'], true);

              resetValidityState = _icepick2.default.setIn(resetValidityState, ['fields', localPath.join('.'), 'validity'], initialFieldState.validity);

              resetValidityState = _icepick2.default.setIn(resetValidityState, ['fields', localPath.join('.'), 'errors'], initialFieldState.errors);
            }

            return {
              v: resetValidityState
            };
          }();

          if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
        }

      case _actionTypes2.default.SET_PRISTINE:
        {
          var formIsPristine = void 0;
          var setPristineState = void 0;

          if (!localPath.length) {
            formIsPristine = true;

            setPristineState = _icepick2.default.merge(state, {
              fields: (0, _mapValues2.default)(state.fields, function (field) {
                return _extends({}, field, {
                  dirty: false, // will be deprecated
                  pristine: true
                });
              })
            });
          } else {
            setPristineState = setField(state, localPath, {
              dirty: false, // will be deprecated
              pristine: true
            });

            formIsPristine = (0, _every2.default)((0, _mapValues2.default)(setPristineState.fields, function (field) {
              return field.pristine;
            }));
          }

          return _icepick2.default.merge(setPristineState, {
            dirty: !formIsPristine, // will be deprecated
            pristine: formIsPristine
          });
        }

      case _actionTypes2.default.SET_UNTOUCHED:
        return setField(state, localPath, {
          touched: false,
          untouched: true });

      // will be deprecated
      case _actionTypes2.default.SET_SUBMITTED:
        return setField(state, localPath, {
          pending: false,
          submitted: !!action.submitted,
          submitFailed: false,
          touched: true,
          untouched: false });

      // will be deprecated
      case _actionTypes2.default.SET_SUBMIT_FAILED:
        return setField(state, localPath, {
          pending: false,
          submitted: false,
          submitFailed: true,
          touched: true,
          untouched: false });

      // will be deprecated
      case _actionTypes2.default.SET_INITIAL:
      case _actionTypes2.default.RESET:
        if (!localPath.length) {
          return localInitialFormState;
        }

        return resetField(state, localPath);

      case _actionTypes2.default.SET_VIEW_VALUE:
        return setField(state, localPath, {
          viewValue: action.value
        });

      default:
        return state;
    }
  };

  return formReducer;
}

function createFormReducer() {
  console.warn('The createFormReducer() function is deprecated (renamed). ' + 'Please use formReducer().');
  /* eslint-disable */
  return _createformReducer.apply(undefined, arguments);
  /* eslint-enable */
}

exports.createFormReducer = createFormReducer;
exports.formReducer = _createFormReducer;
exports.initialFieldState = initialFieldState;
exports.initialFormState = initialFormState;
exports.getField = getField;