'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFieldFromState = exports.getForm = exports.isInvalid = exports.isValid = exports.getValidity = exports.getValue = exports.getEventValue = exports.isTouched = exports.isPristine = exports.isMulti = exports.isFocused = undefined;

var _endsWith = require('lodash/endsWith');

var _endsWith2 = _interopRequireDefault(_endsWith);

var _mapValues = require('lodash/mapValues');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _every = require('lodash/every');

var _every2 = _interopRequireDefault(_every);

var _some = require('lodash/some');

var _some2 = _interopRequireDefault(_some);

var _findKey = require('lodash/findKey');

var _findKey2 = _interopRequireDefault(_findKey);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _toPath = require('lodash/toPath');

var _toPath2 = _interopRequireDefault(_toPath);

var _formReducer = require('../reducers/form-reducer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function isMulti(model) {
  return (0, _endsWith2.default)(model, '[]');
}

function isFocused(field) {
  return field && field.focus;
}

function isPristine(field) {
  return field ? field.pristine : _formReducer.initialFieldState.pristine;
}

function isTouched(field) {
  return field ? field.touched : _formReducer.initialFieldState.touched;
}

function isEvent(event) {
  return !!(event && event.stopPropagation && event.preventDefault);
}

function getEventValue(event) {
  if (!event.target) {
    if (!event.nativeEvent) {
      return undefined;
    }

    return event.nativeEvent.text;
  }

  if (event.target.multiple) {
    return [].concat(_toConsumableArray(event.target.selectedOptions)).map(function (option) {
      return option.value;
    });
  }

  return event.target.value;
}

function getValue(value) {
  return isEvent(value) ? getEventValue(value) : value;
}

function getForm(state, model) {
  var path = model.split('.');
  var modelRoot = path.length === 1 ? state : (0, _get2.default)(state, path.slice(0, path.length - 1));
  var formStateKey = (0, _findKey2.default)(modelRoot, { model: model });
  return modelRoot && modelRoot[formStateKey];
}

function getFieldFromState(state, model) {
  var form = getForm(state, (0, _toPath2.default)(model)[0]);

  if (!form) return null;

  return (0, _formReducer.getField)(form, (0, _toPath2.default)(model).slice(1));
}

function getValidity(validators, value) {
  var modelValue = getValue(value);

  if (typeof validators === 'function') {
    return validators(modelValue);
  }

  return (0, _mapValues2.default)(validators, function (validator) {
    return getValidity(validator, modelValue);
  });
}

function isValid(validity) {
  if ((0, _isPlainObject2.default)(validity)) {
    return (0, _every2.default)(validity, isValid);
  }

  return !!validity;
}

function isInvalid(errors) {
  if ((0, _isPlainObject2.default)(errors)) {
    return (0, _some2.default)(errors);
  }

  return !errors;
}

exports.isFocused = isFocused;
exports.isMulti = isMulti;
exports.isPristine = isPristine;
exports.isTouched = isTouched;
exports.getEventValue = getEventValue;
exports.getValue = getValue;
exports.getValidity = getValidity;
exports.isValid = isValid;
exports.isInvalid = isInvalid;
exports.getForm = getForm;
exports.getFieldFromState = getFieldFromState;